# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from gtk import STOCK_YES, STOCK_NO, ICON_SIZE_BUTTON
from gtk.glade import bindtextdomain, textdomain, XML
from gtk.gdk import pixbuf_new_from_file
from winkirip.lib import build_path, winki_path, check_widgets
from winkirip.page_license import PageLicense
from winkirip.page_source import PageSource
from winkirip.page_encoding import PageEncoding
from winkirip.page_info import PageInfo
from winkirip.page_description import PageDescr
from winkirip.page_size import PageSize
from winkirip.page_title import PageTitle
from winkirip.page_audio import PageAudio
from winkirip.page_subtitles import PageSubtitles
from winkirip.page_rip import PageRip
from winkirip.page_crop import PageCrop
from winkirip.page_tree import PageTree
from winkirip.container import CONTAINER_UNKNOWN, CONTAINER_DVD, CONTAINER_VCD
from winkirip.container import CONTAINER_SVCD, CONTAINER_FILE, CONTAINER_STREAMS
from winkirip.select_settings import SelectSettings
from winkirip.popup_tray_menu import PopupTrayMenu
import gettext

class Druid:
    def __init__(self, app):
        self.__app = app

        self.__xml = None
        self.__load_xml()

        self.__connect_signals()

        self.__current_page = None
        self.__pages = {}
        self.__create_pages()

        self.__check_widgets()

        self.__stop_light = {}
        self.__load_stop_light()

        self.__load_page()

        self.__settings = None
        self.__visible = True
        self.__xpos = 0
        self.__ypos = 0
        self.__window_width = 0
        self.__window_height = 0
        return

    # -- signal handlers --

    def on_help(self, widget, event = None):
        return True

    def on_settings(self, widget, event = None):
        if not self.__settings:
            cfg = self.__app.get_project().get_config()
            toolchecker = self.__app.get_project().get_tool_checker()
            self.__settings = SelectSettings(cfg, toolchecker)

        self.__settings.show()
        return True

    def on_cancel(self, widget, event = None):
        can_exit = True

        obj, prev, next = self.__pages[self.__current_page]
        if obj:
            can_exit = obj.can_exit()

        if not can_exit:
            return False

        self.__app.cleanup()
        return True

    def on_show_hide(self):
        if not self.__app.has_tray():
            return

        wnd = self.get_window()

        if self.__visible:
            self.__store_window_position()
            self.__visible = False
            wnd.hide()

        else:
            self.__restore_window_position()
            self.__visible = True
            wnd.show()
        return

    def on_show_tray_menu(self, widget, event = None):
        if self.__visible:
            return

        popup = PopupTrayMenu(widget)
        popup.set_settings_callback(self.on_settings)
        popup.set_quit_callback(self.on_cancel)

        popup.show(event.button, event.time)
        return

    def on_next(self, widget, event = None):
        obj, prev, next = self.__pages[self.__current_page]
        if obj and not obj.on_next():
            return True

        self.__unload_page()

        name = self.__get_next_page(self.__current_page)
        if not name:
            self.__load_page()
            return True

        self.__current_page = name
        self.__load_page()
        return True

    def on_prev(self, widget, event = None):
        obj, prev, next = self.__pages[self.__current_page]
        if obj and not obj.on_prev():
            return True

        self.__unload_page()

        name = self.__get_prev_page(self.__current_page)
        if not name:
            self.__load_page()
            return True

        self.__current_page = name
        self.__load_page()
        return True

    # -- public interface --

    def set_stoplight(self, light):
        pixbuf = None
        if light == 'go' or light == 'stop':
            pixbuf = self.__stop_light[light]
        else:
            pixbuf = self.__stop_light['empty']

        stoplight = self.__xml.get_widget('image_stop_light')
        stoplight.set_from_pixbuf(pixbuf)
        return

    def set_window_title(self, title):
        wnd = self.get_window()
        wnd.set_title(title)
        return

    def get_window(self):
        """ used by the application object """

        return self.__xml.get_widget('druid')

    def invalidate_after(self, start_page):
        cur_page = self.__get_next_page(start_page)
        while cur_page:
            obj, prev, next = self.__pages[cur_page]
            obj.invalidate()
            cur_page = self.__get_next_page(cur_page)
        return

    def enable_buttons(self, enable, button_list):
        for button in button_list:
            widget = self.__xml.get_widget(button)
            widget.set_sensitive(enable)
        return


    # -- helpers --

    def __no_quit(self, button, event = None):
        self.on_show_hide()
        return True

    def __load_page(self):
        obj, prev, next = self.__pages[self.__current_page]
        if not obj:
            return

        cur_page = obj.get_content()
        target_widget = self.__xml.get_widget('target')
        target_widget.add(cur_page)

        icon_widget = self.__xml.get_widget('image_icon')
        fname = obj.get_icon()
        icon_file = build_path([winki_path, 'glade', fname])
        icon_widget.set_from_file(icon_file)

        title_widget = self.__xml.get_widget('label_title')
        title_text = obj.get_title()
        markup = '<span weight="ultrabold" size="xx-large">%s</span>'
        title_widget.set_markup(markup % title_text)

        obj.on_prepare()

        # setup the navitation buttons
        button_prev = self.__xml.get_widget('button_back')
        button_prev.set_sensitive(prev != None)

        button_next = self.__xml.get_widget('button_next')
        button_next.set_sensitive(next != None)
        return

    def __unload_page(self):
        obj, prev, next = self.__pages[self.__current_page]
        if not obj:
            return

        cur_page = obj.get_content()
        target_widget = self.__xml.get_widget('target')
        target_widget.remove(cur_page)

        return

    def __load_xml(self):
        appname = 'winki'
        i18ndir = build_path([winki_path, 'i18n'])
        gettext.bindtextdomain(appname, i18ndir)
        gettext.textdomain(appname)
        gettext.install(appname, i18ndir)

        try:
            bindtextdomain(appname, i18ndir)
            textdomain(appname)
            xml_path = build_path([winki_path, 'glade', 'druid.glade'])
            self.__xml = XML(xml_path, 'druid', gettext.textdomain())
        except:
            print 'Druid.__load_xml(): Cannot load glade UI file'

        if not self.__xml:
            raise SystemExit

        #self.__get_window_postion_from_config()
        return

    def __connect_signals(self):
        signals = { 'on_button_quit_clicked' : self.on_cancel,
            'on_button_next_clicked' : self.on_next,
            'on_button_back_clicked' : self.on_prev,
            'on_button_settings_clicked' : self.on_settings,
            'on_button_help_clicked' : self.on_help}

        self.__xml.signal_autoconnect(signals)

        check_widgets(self.__xml, ['druid'])
        wnd = self.get_window()
        wnd.connect('destroy', self.__no_quit)
        wnd.connect('delete_event', self.__no_quit)
        return

    def __load_stop_light(self):
        widget = self.__xml.get_widget('button_quit')

        #fname = build_path([winki_path, 'glade', 'geher.png'])
        pixbuf = widget.render_icon(STOCK_YES, ICON_SIZE_BUTTON)
        self.__stop_light['go'] = pixbuf

        #fname = build_path([winki_path, 'glade', 'steher.png'])
        pixbuf = widget.render_icon(STOCK_NO, ICON_SIZE_BUTTON)
        self.__stop_light['stop'] = pixbuf

        fname = build_path([winki_path, 'glade', 'empty.png'])
        pixbuf = pixbuf_new_from_file(fname)
        self.__stop_light['empty'] = pixbuf
        return

    def __get_next_page(self, name):
        source_type = CONTAINER_UNKNOWN
        dest_type = CONTAINER_UNKNOWN
        sources = []
        if self.__app.get_project():
            source_type = self.__app.get_project().get_source_type()
            dest_type = self.__app.get_project().get_dest_type()
            sources = self.__app.get_project().get_source()

        # more than one source containers
        if len(sources) > 1:

            # use tree instead of audio and subtitle
            if name == 'description':
                return 'tree'

        # exactly one source container
        else:
            if source_type == CONTAINER_DVD:
                if name == 'title':
                    return 'audio'

                elif name == 'audio' and not self.__source_has_subtitles():
                    return 'size'

            #if source_type == CONTAINER_FILE:
            #   if name == 'info':
            #       return 'audio'

            #   elif name == 'subtitles' and encoding == 'copy':
            #       return 'crop'

            if dest_type == CONTAINER_VCD or dest_type == CONTAINER_SVCD:
                if name == 'info':
                    return 'audio'

                elif name == 'audio' and encoding == 'copy':
                    return 'crop'

                elif name == 'audio':
                    return 'size'

        obj, prev, next = self.__pages[name]
        return next

    def __get_prev_page(self, name):
        source_type = CONTAINER_UNKNOWN
        dest_type = CONTAINER_UNKNOWN
        sources = []
        if self.__app.get_project():
            source_type = self.__app.get_project().get_source_type()
            dest_type = self.__app.get_project().get_dest_type()
            sources = self.__app.get_project().get_source()

        # more than one source container
        if len(sources) > 1:

            # use tree instead of subtitle and audio
            if name == 'size':
                return 'tree'

        # exactly one source container
        else:
            if source_type == CONTAINER_DVD:
                if name == 'audio':
                    return 'title'

                elif name == 'size' and not self.__source_has_subtitles():
                    return 'audio'

            #if source_type == CONTAINER_FILE:
            #   if encoding == 'copy' and name == 'crop':
            #       return 'subtitles'

            #   elif name == 'audio':
            #       return 'info'

            if dest_type == CONTAINER_VCD or dest_type == CONTAINER_SVCD:
                if encoding == 'copy' and name == 'crop':
                    return 'audio'

                elif name == 'audio':
                    return 'info'

                elif name == 'size':
                    return 'audio'

        obj, prev, next = self.__pages[name]
        return prev

    def __create_pages(self):
        obj = PageLicense('start.glade', self.__app, self)
        prev = None
        next = 'source'
        self.__pages['license'] = [obj, prev, next]

        obj = PageSource('source.glade', self.__app, self)
        prev = 'license'
        next = 'encoding'
        self.__pages['source'] = [obj, prev, next]

        obj = PageEncoding('encoding.glade', self.__app, self)
        prev = 'source'
        next = 'info'
        self.__pages['encoding'] = [obj, prev, next]

        obj = PageInfo('info.glade', self.__app, self)
        prev = 'encoding'
        next = 'title'
        self.__pages['info'] = [obj, prev, next]

        obj = PageTitle('title.glade', self.__app, self)
        prev = 'info'
        next = 'description'
        self.__pages['title'] = [obj, prev, next]

        obj = PageDescr('description.glade', self.__app, self)
        prev = 'title'
        next = 'audio'
        self.__pages['description'] = [obj, prev, next]

        obj = PageAudio('audio.glade', self.__app, self)
        prev = 'description'
        next = 'subtitles'
        self.__pages['audio'] = [obj, prev, next]

        obj = PageSubtitles('subtitles.glade', self.__app, self)
        prev = 'audio'
        next = 'size'
        self.__pages['subtitles'] = [obj, prev, next]

        obj = PageSize('size.glade', self.__app, self)
        prev = 'subtitles'
        next = 'crop'
        self.__pages['size'] = [obj, prev, next]

        obj = PageCrop('crop.glade', self.__app, self)
        prev = 'size'
        next = 'rip'
        self.__pages['crop'] = [obj, prev, next]

        obj = PageRip('rip.glade', self.__app, self)
        prev = 'crop'
        next = None
        self.__pages['rip'] = [obj, prev, next]
    
        # additional page (shows up between info and size instead of title, audio and subtitle)
        obj = PageTree('tree.glade', self.__app, self)
        prev = 'description'
        next = 'size'
        self.__pages['tree'] = [obj, prev, next]

        self.__current_page = 'license'
        return

    def __check_widgets(self):
        widgets = []
        for page_name in self.__pages:
            widgets.append(page_name)

        for page_name in self.__pages:
            obj, prev, next = self.__pages[page_name]
            if not obj:
                continue
            obj.check_widgets()
        return

    def __store_window_position(self):
        wnd = self.get_window()
        (self.__xpos, self.__ypos) = wnd.get_position()
        (self.__window_width, self.__window_height) = wnd.get_size()

        #self.__save_window_postion_to_config()
        return

    def __restore_window_position(self):
        wnd = self.get_window()
        wnd.move(self.__xpos, self.__ypos)
        wnd.set_size_request(self.__window_width, self.__window_height)
        return

    def __get_window_postion_from_config(self):
        cfg = self.__app.get_project().get_config()
        self.__xpos, self.__ypos, self.__window_width, self.__window_height = cfg.get_window_position()
        return

    def __save_window_postion_to_config(self):
        cfg = self.__app.get_project().get_config()
        cfg.set_window_position(self.__xpos, self.__ypos, self.__window_width, self.__window_height)
        cfg.write()
        return

    def __source_has_subtitles(self):
        source_containers = self.__app.get_project().get_source()
        dest_containers = self.__app.get_project().get_dest()
        if not dest_containers or not len(dest_containers) or not source_containers or not len(source_containers):
            return False

        # this function assumes that the source title id == dest title id:
        # it can't give reliable information for more than one source/dest container
        if len(source_containers) > 1 or len(dest_containers) > 1:
            return True

        for dest_title in dest_containers[0]:
            for src_title in source_containers[0]:
                if src_title.get_id() == dest_title.get_id():
                    if len(dest_title.get_subtitles()) > 0:
                        return True

        return False

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
