# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from gtk.gdk import pixbuf_new_from_file, _2BUTTON_PRESS
from winkirip.lib import winki_path, build_path, debug
from winkirip.page_base import BasePage
from winkirip.page_base_two_trees import BaseTwoTreesPage, COLUMN_LANG_OBJECT

class PageAudio(BasePage, BaseTwoTreesPage):
    def __init__(self, xml, app, druid):
        BasePage.__init__(self, xml, app, druid)
        BaseTwoTreesPage.__init__(self, 'language')
        return
        
    # -- methods inherited from BasePage --

    def get_icon(self):
        return 'page_audio.png'

    def get_title(self):
        return _('Audio Selection')

    def get_signal_list(self):
        return {'on_treeview_button_press_event': self.on_treeview_button_pressed}

    def get_widget_list(self):
        return []

    def on_prepare(self):
        self.set_stoplight('go')

        if not self.get_needs_init():
            return True

        self.set_treeviews(self.get_widget('treeview_source'), self.get_widget('treeview_dest'))

        self.__set_dest_container()
        self.__init_treeview('treeview_source', _('Available audio languages'))

        self.__reset_dest_container()
        self.__init_treeview('treeview_dest', _('Audio languages in use'))

        self.set_needs_init(False)
        return True

    def on_next(self):
        return self.update_project()

    def on_prev(self):
        return True

    def can_exit(self):
        return True

    def update_project(self):
        self.disable_buttons()

        treeview = self.get_widget('treeview_dest')
        model = treeview.get_model()

        if not self.__check_selected_streams(model):
            self.enable_buttons()
            return False

        self.__sumbit_selected_streams(model)
        self.get_druid().invalidate_after('audio')
        self.enable_buttons()
        return True

    def show_help(self):
        return BasePage.show_help(self)

    # -- signal handlers --

    def on_treeview_button_pressed(self, treeview, event):
        result = False

        # double click with the left mouse button
        if event.button == 1 and event.type == _2BUTTON_PRESS:
            self.move_selected(treeview)
            self.get_druid().invalidate_after('subtitles')
            result = True

        return result

    def on_drag_data_received(self, treeview, context, x, y, selection, info, timestamp):
        self.drag_data_receive(treeview, x, y, selection)
        self.get_druid().invalidate_after('subtitles')
        return True

    def on_drag_data_get(self, treeview, context, selection, info, timestamp):
        return self.drag_data_get(treeview, selection)

    # -- public interface --

    # -- helpers --

    def __init_treeview(self, treeview_name, treeview_title):
        treeview = self.get_widget(treeview_name)
        model = treeview.get_model()
        if not model:
            self.create_treeview(treeview, treeview_title)
            self.setup_dnd(treeview, self.on_drag_data_get, self.on_drag_data_received)
        else:
            model.clear()

        title_list = self.get_project().get_dest()[0]
        self.__fill_model(title_list, treeview)
        return

    def __fill_model(self, title_list, treeview):
        model = treeview.get_model()

        title_fname = build_path([winki_path, 'glade', 'media_title.png'])
        title_pixbuf = pixbuf_new_from_file(title_fname)

        iso_codes = self.get_project().get_iso_codes()

        for title in title_list:
            title_name = ('<span weight="bold">%s</span>' % title.get_name())
            iterator = model.append(None, [title_pixbuf, title_name, title])
            counter = 1
            for audio in title.get_audio():
                lang = iso_codes.find_iso_639_1(audio.get_language())
                if not lang:
                    debug('PageAudio.__fill_model()', 'unknown language %s' % audio.get_language())
                    continue

                audio_name = lang.get_name()
                if not audio_name or not len(audio_name):
                    audio_name = (_('Audio Stream %d') % counter)

                audio_name = ('<span weight="normal">%s</span>' % audio_name)
                model.append(iterator, [lang.get_flag(), audio_name, audio])
                counter += 1

        treeview.expand_all()
        return

    def __set_dest_container(self):
        dest_titles = self.get_project().get_dest()[0]
        for title in dest_titles:
            astreams = filter(lambda astream: True, title.get_audio())
            title.reset_audio()

            for astream in astreams:
                astream.reset_source()
                dest_audio = astream.copy()
                dest_audio.set_source(astream)
                title.add_audio(dest_audio)
        return

    def __reset_dest_container(self):
        dest_titles = self.get_project().get_dest()[0]
        for title in dest_titles:
            title.reset_audio()
        return

    def __check_selected_streams(self, model):
        for row in model:
            if not model.iter_children(row.iter):
                title = model.get_value(row.iter, COLUMN_LANG_OBJECT)
                primary = _('No audio for "%s" selected') % title.get_name()
                secondary = _('You won\'t be able to hear sound in this title if you choose to continue')
                affirmative = _('Continue without audio')
                result = self.show_alert(primary, secondary, affirmative)
                if not result:
                    return False

        return True

    def __sumbit_selected_streams(self, model):
        dest_titles = self.get_project().get_dest()[0]
        for row in model:
            flag, text, source_title = row
            for dest_title in dest_titles:
                if dest_title.get_id() == source_title.get_id():
                    result = []
                    iterator = model.iter_children(row.iter)
                    if not iterator:
                        continue

                    while iterator:
                        audio = model.get_value(iterator, COLUMN_LANG_OBJECT)
                        result.append(audio)
                        iterator = model.iter_next(iterator)

                    dest_title.reset_audio()
                    for audio in result:
                        dest_title.add_audio(audio)
        return


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
