# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from gtk import CellRendererPixbuf, CellRendererText, TreeViewColumn
from gtk import TreeStore, SELECTION_MULTIPLE, TreeRowReference
from gtk import TREE_VIEW_DROP_BEFORE, TREE_VIEW_DROP_INTO_OR_BEFORE
from gtk import TREE_VIEW_DROP_INTO_OR_AFTER, TREE_VIEW_DROP_AFTER
from gtk.gdk import Pixbuf, ACTION_DEFAULT
from gtk.gdk import ACTION_MOVE, ACTION_COPY, BUTTON1_MASK
from gobject import TYPE_STRING, TYPE_PYOBJECT
from copy import deepcopy
from winkirip.container import Container
from winkirip.container_title import Title
from winkirip.container_video import Video
from winkirip.container_audio import Audio
from winkirip.container_subtitle import Subtitle
from winkirip.lib import treepath_tuple_to_string, treepath_string_to_tuple
from winkirip.lib import debug, error


COLUMN_LANG_PIXBUF = 0
COLUMN_LANG_TITLE = 1
COLUMN_LANG_OBJECT = 2

COLUMN_TREE_LANG = 0
COLUMN_TREE_TYPE = 1
COLUMN_TREE_TITLE = 2
COLUMN_TREE_OBJECT = 3

# containers are the first level in the tree, titles - the second and chapters - the third
CONT_PATH_LEN = 1
TITLE_PATH_LEN = 2
STREAM_PATH_LEN = 3

class BaseTwoTreesPage:
    def __init__(self, kind):
        self.__source_treeview = None
        self.__dest_treeview = None
        self.__kind = kind
        return
        
    # -- public interface --

    def set_treeviews(self, source_treeview, dest_treeview):
        self.__source_treeview = source_treeview
        self.__dest_treeview = dest_treeview
        return

    def create_treeview(self, treeview, header_text):
        model = TreeStore(Pixbuf, TYPE_STRING, TYPE_PYOBJECT)
        treeview.set_model(model)
        treeview.set_rules_hint(True)

        renderer = CellRendererPixbuf()
        renderer.set_property('xalign', 0)
        column = TreeViewColumn('', renderer, pixbuf = COLUMN_LANG_PIXBUF)
        treeview.append_column(column)

        renderer = CellRendererText()
        column = TreeViewColumn(header_text, renderer, markup = COLUMN_LANG_TITLE)
        treeview.append_column(column)

        selection = treeview.get_selection()
        selection.set_mode(SELECTION_MULTIPLE)

        treeview.set_reorderable(True)
        return

    def setup_dnd(self, treeview, get_callback, received_callback):
        flags = ACTION_DEFAULT | ACTION_MOVE | ACTION_COPY
        treeview.enable_model_drag_dest([('text/plain', 0, 0)], flags)
        treeview.connect("drag-data-received", received_callback)

        treeview.enable_model_drag_source(BUTTON1_MASK, [('text/plain', 0, 0)], flags)
        treeview.connect("drag-data-get", get_callback)
        return

    def drag_data_get(self, treeview, selection):
        treeselection = treeview.get_selection()
        (model, pathlist) = treeselection.get_selected_rows()
        data = ''
        for path in pathlist:
            string_path = treepath_tuple_to_string(path)
            data += '%s\t%s\n' % (treeview.get_name(), string_path)
        selection.set('text/plain', 8, data)
        return True

    def drag_data_receive(self, treeview, x, y, selection):
        drop_info = treeview.get_dest_row_at_pos(x, y)

        dest_tv_name = treeview.get_name()
        paths, source_tv_name = self.__filter_paths(dest_tv_name, selection.data, drop_info)
        if not len(paths):
            return True

        self.__move_streams(paths, source_tv_name, dest_tv_name, drop_info)
        return True

    def move_selected(self, treeview):
        source_tv_name = treeview.get_name()
        dest_tv_name = self.__source_treeview.get_name()
        if self.__source_treeview.get_name() == source_tv_name:
            dest_tv_name = self.__dest_treeview.get_name()

        paths = []
        treeselection = treeview.get_selection()
        (model, paths) = treeselection.get_selected_rows()

        self.__move_streams(paths, source_tv_name, dest_tv_name, None)
        return

    # -- helpers --

    def __filter_paths(self, dest_tv_name, data, drop_info):
        result = []

        source_items = data.split('\n')
        if not len(source_items):
            return result, None

        # ignore external sources for now
        source_name = self.__source_treeview.get_name()
        dest_name = self.__dest_treeview.get_name()
        tmp = source_items[0].split('\t')
        if len(tmp) != 2 or (tmp[0] != source_name and tmp[0] != dest_name):
            return result, None

        dest_path = None
        dest_pos = None
        if drop_info:
            dest_path, dest_pos = drop_info

        source_tv_name = None
        for source_item in source_items:
            if not source_item or not len(source_item):
                continue

            source_tv_name, source_path = source_item.split('\t')

            if self.__kind == 'language':
                if self.__validate_lang_move(source_tv_name, source_path, dest_tv_name, dest_path):
                    result.append(source_path)

            elif self.__kind == 'tree':
                if self.__validate_tree_move(source_tv_name, source_path, dest_tv_name, dest_path, dest_pos, source_items):
                    result.append(source_path)

            else:
                debug('BaseTwoTreesPage.__filter_paths()', 'unknown kind: %s' % self.__kind, error)

        return result, source_tv_name

    def __validate_lang_move(self, source_tv_name, source_path, dest_tv_name, dest_path):
        source_treeview = self.__dest_treeview
        if self.__source_treeview.get_name() == source_tv_name:
            source_treeview = self.__source_treeview
        source_model = source_treeview.get_model()

        source_iter = source_model.get_iter(source_path)
        source_object = source_model.get_value(source_iter, COLUMN_LANG_OBJECT)

        # titles could not be rearanged or moved
        if isinstance(source_object, Title):
            return False

        # dropping into the other treeview is always permitted
        if source_tv_name != dest_tv_name:
            return True

        # rearrange is possible only inside a title
        return self.__same_title(source_path, dest_path)

    def __validate_tree_move(self, source_tv_name, source_path, dest_tv_name, dest_path, dest_pos, source_items):
        if not dest_path or not len(dest_path):
            return False

        source_treeview = self.__dest_treeview
        if self.__source_treeview.get_name() == source_tv_name:
            source_treeview = self.__source_treeview
        source_model = source_treeview.get_model()

        source_iter = source_model.get_iter(source_path)
        source_object = source_model.get_value(source_iter, COLUMN_TREE_OBJECT)

        # containers could not be rearanged or moved
        if isinstance(source_object, Container):
            return False

        # dropping from dest into source treeview is not permitted
        if source_tv_name != dest_tv_name and source_tv_name == 'treeview_dest':
            return False

        # rearanging objects is the source treeview is not permitted
        if source_tv_name == dest_tv_name and source_tv_name == 'treeview_source':
            return False

        # rearanging titles is meaningfull only in the dest_treeview 
        if isinstance(source_object, Title) and source_tv_name != 'treeview_dest':
            return False

        # all the dragged objects have to have the same path length (should be from the same tree level)
        # with the exception that it is possible to drop streams over a title
        path_len = len(dest_path)
        for sitem in source_items:
            if not sitem or len(sitem) < 1:
                continue

            tv_name, srcpath = sitem.split('\t')
            spath = treepath_string_to_tuple(srcpath)
            if len(spath) != path_len and len(spath) != STREAM_PATH_LEN and path_len != TITLE_PATH_LEN:
                return False

        return True

    def __move_to_same_tv(self, source_paths, source_tv_name, drop_info):
        treeview = None
        if self.__source_treeview.get_name() == source_tv_name:
            treeview = self.__source_treeview
        else:
            treeview = self.__dest_treeview
        model = treeview.get_model()
        dest_path, dest_pos = drop_info
        dest_ref = TreeRowReference(model, dest_path)
        dest_iter = model.get_iter(dest_path)
        parent_iter = model.iter_parent(dest_iter)

        # collect references to the source rows
        source_paths = self.__paths_to_tuple(source_paths, dest_pos)
        source_refs = []
        for source_path in source_paths:
            # don't drop over the selection: this would invalidate our
            # precious dest_ref
            if source_path == dest_path:
                return

            source_refs.append(TreeRowReference(model, source_path))

        # move and collect references to the inserted rows
        select_refs = []
        for source_ref in source_refs:
            source_path = source_ref.get_path()
            source_iter = model.get_iter(source_path)
            value_list = self.__get_values(model, source_iter)

            # if we are about to move a title - we have to move all associated streams too
            stream_refs = []
            if self.__kind == 'tree' and len(source_path) == TITLE_PATH_LEN:
                stream_iter = model.iter_children(source_iter)
                while stream_iter:
                    stream_refs.append(self.__get_values(model, stream_iter))
                    stream_iter = model.iter_next(stream_iter)

            # disallow moving streams to titles
            if self.__kind == 'tree' and len(source_path) == STREAM_PATH_LEN and len(dest_ref.get_path()) == TITLE_PATH_LEN:
                return

            model.remove(source_iter)

            dest_path = dest_ref.get_path() 
            dest_iter = model.get_iter(dest_path)
            if dest_pos == TREE_VIEW_DROP_BEFORE or dest_pos == TREE_VIEW_DROP_INTO_OR_BEFORE:
                it = model.insert_before(parent_iter, dest_iter, value_list)
                select_refs.append(TreeRowReference(model, model.get_path(it)))
            else:
                it = model.insert_after(parent_iter, dest_iter, value_list)
                select_refs.append(TreeRowReference(model, model.get_path(it)))

            for stream_ref in stream_refs:
                model.append(it, stream_ref)

        # select the inserted rows
        selection = treeview.get_selection()
        selection.unselect_all()
        for select_ref in select_refs:
            selection.select_path(select_ref.get_path())

        # shift the focus
        treeview.grab_focus()
        return

    def __move_to_other_tv(self, source_paths, source_tv_name, drop_info):
        source_tv, dest_tv, source_model, dest_model = self.__get_treeview_objects(source_tv_name)

        dest_path = None
        dest_pos = None
        if drop_info:
            dest_path, dest_pos = drop_info
            iterator = dest_model.get_iter(dest_path)

            # dropping on a title results in appending to the title
            if self.__kind == 'language' and isinstance(dest_model.get_value(iterator, COLUMN_LANG_OBJECT), Title):
                dest_pos = None

            if self.__kind == 'tree' and isinstance(dest_model.get_value(iterator, COLUMN_TREE_OBJECT), Title):
                dest_pos = None

        # collect references to the source rows
        source_refs = []
        for source_path in source_paths:
            source_refs.append(TreeRowReference(source_model, source_path))
            
            # dropping into antoher title results in appending to the right one
            # if we move streams in "page_audio" or "page_subtitle";
            if self.__kind == 'language' and not self.__same_title(source_path, dest_path):
                dest_pos = None

        # move and collect references to the inserted rows
        select_refs = []
        for source_ref in source_refs:
            source_iter = source_model.get_iter(source_ref.get_path())

            dest_pos, dest_iter = self.__get_dest_iter(source_iter, source_model, dest_model, dest_pos, dest_path)
            if not dest_iter:
                debug('BaseTwoTreesPage.__move_to_other_tv()', 'no destination iterator found: skipping')
                continue

            parent_iter = dest_model.iter_parent(dest_iter)
            value_list = self.__get_values(source_model, source_iter, True) # this cludge with the "True" will bite back...

            if not dest_pos:
                it = dest_model.append(dest_iter, value_list)
                select_refs.append(TreeRowReference(dest_model, dest_model.get_path(it)))

            elif dest_pos == TREE_VIEW_DROP_BEFORE or dest_pos == TREE_VIEW_DROP_INTO_OR_BEFORE:
                it = dest_model.insert_before(parent_iter, dest_iter, value_list)
                select_refs.append(TreeRowReference(dest_model, dest_model.get_path(it)))

            else:
                it = dest_model.insert_after(parent_iter, dest_iter, value_list)
                select_refs.append(TreeRowReference(dest_model, dest_model.get_path(it)))

            # remove from the source tree if we move streams in "page_audio" or "page_subtitle"
            # or the user (re)moves objects from the dest treeview
            if self.__kind == 'language' or source_tv_name == 'treeview_dest':
                source_path = source_ref.get_path()
                source_model.remove(source_model.get_iter(source_path)) 
                if not parent_iter:
                    dest_tv.expand_row(dest_model[0].path, True)
                else:
                    dest_tv.expand_row(dest_model.get_path(parent_iter), True)

        # unselect the source treeview
        selection = source_tv.get_selection()
        selection.unselect_all()

        # select the inserted rows
        selection = dest_tv.get_selection()
        selection.unselect_all()
        for select_ref in select_refs:
            selection.select_path(select_ref.get_path())

        # shift the focus
        dest_tv.grab_focus()
        return

    def __get_dest_iter(self, source_iter, source_model, dest_model, dest_pos, dest_path):
        if self.__kind == 'language':
            return self.__get_dest_iter_language(source_iter, source_model, dest_model, dest_pos, dest_path)

        elif self.__kind == 'tree':
            return self.__get_dest_iter_tree(source_iter, source_model, dest_model, dest_pos, dest_path)

        debug('BaseTwoTreesPage.__get_dest_iter()', 'unknown kind: %s' % self.__kind, error)
        return None

    def __get_dest_iter_language(self, source_iter, source_model, dest_model, dest_pos, dest_path):
        parent_iter = source_model.iter_parent(source_iter)
        source_title = source_model.get_value(parent_iter, COLUMN_LANG_OBJECT)

        for dest_row in dest_model:
            dest_title = dest_row[COLUMN_LANG_OBJECT]

            # destination title found
            if dest_title.get_id() == source_title.get_id():
                if not dest_pos:
                    return dest_pos, dest_row.iter
                else:
                    return dest_pos, dest_model.get_iter(dest_path)

        debug('BaseTwoTreesPage.__get_dest_iter_language()', 'no dest iter found', error)
        return None, None

    def __get_dest_iter_tree(self, source_iter, source_model, dest_model, dest_pos, dest_path):
        dest_iter = dest_model.get_iter(dest_path)
        dest_object = dest_model.get_value(dest_iter, COLUMN_TREE_OBJECT)
        if not dest_object:
            return None, None

        # don't allow streams to be dragged into a container
        if isinstance(dest_object, Container):
            return None, None

        # if the streams are dragged into a title - append them at the end
        if isinstance(dest_object, Title):
            return dest_pos, dest_iter

        if isinstance(dest_object, Audio) or isinstance(dest_object, Video) or isinstance(dest_object, Subtitle):
            if dest_pos == TREE_VIEW_DROP_BEFORE or TREE_VIEW_DROP_INTO_OR_BEFORE:
                dest_pos = TREE_VIEW_DROP_INTO_OR_BEFORE
            else:
                dest_pos = TREE_VIEW_DROP_INTO_OR_AFTER
            return dest_pos, dest_iter

        return None, None

    def __get_treeview_objects(self, source_tv_name):
        source_tv = None
        if self.__source_treeview.get_name() == source_tv_name:
            source_tv = self.__source_treeview
        else:
            source_tv = self.__dest_treeview
        source_model = source_tv.get_model()

        dest_tv = None
        if self.__source_treeview.get_name() == source_tv_name:
            dest_tv = self.__dest_treeview
        else:
            dest_tv = self.__source_treeview
        dest_model = dest_tv.get_model()
        return source_tv, dest_tv, source_model, dest_model

    def __move_streams(self, source_paths, source_tv_name, dest_tv_name, drop_info):
        if source_tv_name == dest_tv_name:
            self.__move_to_same_tv(source_paths, source_tv_name, drop_info)
        else:
            self.__move_to_other_tv(source_paths, source_tv_name, drop_info)
        return

    def __reverse_paths(self, paths):
        result = []
        for path in paths:
            result.append(treepath_string_to_tuple(path))

        result.sort()
        result.reverse()
        return result

    def __paths_to_tuple(self, paths, pos):
        result = []

        # reverse and convert
        if pos != TREE_VIEW_DROP_BEFORE and pos != TREE_VIEW_DROP_INTO_OR_BEFORE:
            result = self.__reverse_paths(paths)

        # just convert
        else:
            for path in paths:
                result.append(treepath_string_to_tuple(path))

        return result

    def __get_values(self, model, iter, copy_object = False):
        if self.__kind == 'language':
            return self.__get_values_language(model, iter)

        elif self.__kind == 'tree':
            return self.__get_values_tree(model, iter, copy_object)

        debug('BaseTwoTreesPage.__get_values()', 'unknown kind: %s' % self.__kind, error)
        return []

    def __get_values_language(self, model, iter):
        text = model.get_value(iter, COLUMN_LANG_TITLE)
        obj = model.get_value(iter, COLUMN_LANG_OBJECT)
        pixbuf = model.get_value(iter, COLUMN_LANG_PIXBUF)
        return [pixbuf, text, obj]
        
    def __get_values_tree(self, model, iter, copy_object):
        lang = model.get_value(iter, COLUMN_TREE_LANG)
        ttype = model.get_value(iter, COLUMN_TREE_TYPE)
        text = model.get_value(iter, COLUMN_TREE_TITLE)
        o = model.get_value(iter, COLUMN_TREE_OBJECT)
        if not copy_object:
            obj = o
        else:
            obj = deepcopy(o)
            assert(not obj.get_source())
            obj.set_source(o)

        return [lang, ttype, text, obj]

    def __same_title(self, source_path, dest_path):
        source_tuple = source_path
        if not isinstance(source_tuple, tuple):
            source_tuple = treepath_string_to_tuple(source_path)

        dest_tuple = dest_path
        if not isinstance(dest_tuple, tuple):
            dest_tuple = treepath_string_to_tuple(dest_path)

        if len(source_tuple) != len(dest_tuple):
                return False

        # both paths point to the same title if all but the last elements equal
        for i in range (len(source_tuple) - 1):
            if source_tuple[i] != dest_tuple[i]:
                return False

        return True

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
