# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from gtk.gdk import pixbuf_new_from_file, _2BUTTON_PRESS
from winkirip.lib import winki_path, build_path, debug
from winkirip.page_base import BasePage
from winkirip.page_base_two_trees import BaseTwoTreesPage, COLUMN_LANG_OBJECT


class PageSubtitles(BasePage, BaseTwoTreesPage):
    def __init__(self, xml, app, druid):
        BasePage.__init__(self, xml, app, druid)
        BaseTwoTreesPage.__init__(self, 'language')
        return
        
    # -- methods inherited from page_base.BasePage --

    def get_icon(self):
        return 'page_subtitle.png'

    def get_title(self):
        return _('Subtitles Selection')

    def get_signal_list(self):
        return {'on_checkbutton_mix_subtitles_toggled' : self.on_mix_subtitles,
                'on_treeview_button_press_event': self.on_treeview_button_pressed}

    def get_widget_list(self):
        return []

    def on_prepare(self):
        self.__toggle_stoplight()

        if not self.get_needs_init():
            return

        self.set_treeviews(self.get_widget('treeview_source'), self.get_widget('treeview_dest'))

        self.__set_dest_container()
        self.__init_treeview('treeview_source', _('Available subtitles'))

        self.__reset_dest_container()
        self.__init_treeview('treeview_dest', _('Subtitles in use'))

        self.set_needs_init(False)
        return True

    def on_next(self):
        return self.update_project()

    def on_prev(self):
        return True

    def can_exit(self):
        return True

    def update_project(self):
        self.disable_buttons()

        treeview = self.get_widget('treeview_dest')
        model = treeview.get_model()

        checkbutton = self.get_widget('checkbutton_mix_subtitles')
        if checkbutton.get_active():
            stream_status = self.__check_selected_streams(model)
            if stream_status == 0:
                pass

            elif stream_status == 1:
                primary = _('No subtitles selected')
                secondary = _('You have choosen to mix the subtitles into the movie but havent selected any. If you choose to go ahead there will be no subtitles in the movie.')
                affirmative = _('Continue without subtitles')
                result = self.show_alert(primary, secondary, affirmative)
                if not result:
                    self.enable_buttons()
                    return False

            elif stream_status == 2:
                primary = _('Too many subtitles selected')
                secondary = _('You have choosen to mix the subtitles into the movie. At most one subtitle per title may be mixed into the video.')
                self.show_error(primary, secondary)
                self.enable_buttons()
                return False

            else:
                assert(False)

        self.__sumbit_selected_streams(model)
        self.get_druid().invalidate_after('subtitles')

        self.enable_buttons()
        return True

    def show_help(self):
        return BasePage.show_help(self)

    # -- signal handlers --

    def on_treeview_button_pressed(self, treeview, event):
        result = False

        # double click with the left mouse button
        if event.button == 1 and event.type == _2BUTTON_PRESS:
            self.move_selected(treeview)
            self.get_druid().invalidate_after('size')
            result = True

        return result

    def on_mix_subtitles(self, widget, event = None):
        self.__toggle_stoplight()
        return

    def on_drag_data_received(self, treeview, context, x, y, selection, info, timestamp):
        self.drag_data_receive(treeview, x, y, selection)

        self.__toggle_stoplight()
        self.get_druid().invalidate_after('subtitles')
        return True

    def on_drag_data_get(self, treeview, context, selection, info, timestamp):
        return self.drag_data_get(treeview, selection)

    # -- public interface --

    # -- helpers --

    def __init_treeview(self, treeview_name, treeview_title):
        treeview = self.get_widget(treeview_name)
        model = treeview.get_model()
        if not model:
            self.create_treeview(treeview, treeview_title)
            self.setup_dnd(treeview, self.on_drag_data_get, self.on_drag_data_received)
        else:
            model.clear()

        title_list = self.get_project().get_dest()[0]
        self.__fill_model(title_list, treeview)
        return

    def __fill_model(self, title_list, treeview):
        model = treeview.get_model()

        title_fname = build_path([winki_path, 'glade', 'media_title.png'])
        title_pixbuf = pixbuf_new_from_file(title_fname)

        iso_codes = self.get_project().get_iso_codes()

        for title in title_list:
            title_name = ('<span weight="bold">%s</span>' % title.get_name())
            iterator = model.append(None, [title_pixbuf, title_name, title])
            counter = 1
            for subtitle in title.get_subtitles():
                lang = iso_codes.find_iso_639_1(subtitle.get_language())
                if not lang:
                    debug('PageSubtitles.__fill_model()', 'unknown language %s' % subtitle.get_language())
                    continue

                subtitle_name = lang.get_name()
                if not subtitle_name or not len(subtitle_name):
                    subtitle_name = (_('Subtitle %d') % counter)

                subtitle_name = ('<span weight="normal">%s</span>' % subtitle_name)
                model.append(iterator, [lang.get_flag(), subtitle_name, subtitle])
                counter += 1

        treeview.expand_all()
        return

    def __set_dest_container(self):
        dest_titles = self.get_project().get_dest()[0]
        for title in dest_titles:
            sstreams = filter(lambda sstream: True, title.get_subtitles())
            title.reset_subtitles()

            for sstream in sstreams:
                sstream.reset_source()
                dest_subtitle = sstream.copy()
                dest_subtitle.set_source(sstream)
                title.add_subtitle(dest_subtitle)
        return

    def __reset_dest_container(self):
        dest_titles = self.get_project().get_dest()[0]
        for title in dest_titles:
            title.reset_subtitles()
        return

    def __sumbit_selected_streams(self, model):
        dest_titles = self.get_project().get_dest()[0]
        for row in model:
            flag, text, source_title = row
            for dest_title in dest_titles:
                if dest_title.get_id() == source_title.get_id():
                    result = []
                    iterator = model.iter_children(row.iter)
                    if not iterator:
                        continue

                    while iterator:
                        subtitle = model.get_value(iterator, COLUMN_LANG_OBJECT)
                        result.append(subtitle)
                        iterator = model.iter_next(iterator)

                    dest_title.reset_subtitles()
                    for subtitle in result:
                        dest_title.add_subtitle(subtitle)
        return

    def __toggle_stoplight(self):
        checkbutton = self.get_widget('checkbutton_mix_subtitles')
        if checkbutton.get_active():
            treeview = self.get_widget('treeview_dest')
            model = treeview.get_model()

            if self.__check_selected_streams(model) == 0:
                self.set_stoplight('go')
            else:
                self.set_stoplight('stop')
        else:
            self.set_stoplight('go')
        return

    def __check_selected_streams(self, model):
        checkbutton = self.get_widget('checkbutton_mix_subtitles')

        if checkbutton.get_active():
            for row in model:
                iterator = model.iter_children(row.iter)
                if not iterator:
                    return 1

                # no more than one subtitle per title is allowed in this mode
                if model.iter_next(iterator):
                    return 2

        return 0

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
