# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.container import CONTAINER_UNKNOWN, CONTAINER_DVD, CONTAINER_FILE
from winkirip.container_video import Video, VIDEO_XVID, VIDEO_MPEG2
from winkirip.container_video import VIDEO_MPEG4
from winkirip.container_audio import Audio
from winkirip.container_subtitle import Subtitle
from winkirip.lib import debug, error, to_int, append_to_filename, set_extension
from winkirip.lib import file_size, number_lines, build_path, does_path_exist
from os import unlink

EMPTY_FILE = 0
EMPTY_XML_FILE = 1

class Worker:
    def __init__(self, context, create_callback, ui_name):
        self.__context = context
        self.__create_callback = create_callback
        self.__commands = []
        self.__name = ui_name
        return

    # -- public interface --

    def get_context(self):
        return self.__context

    def get_source_container(self, cont_id):
        source = self.__context.get_project().get_source()
        match_list = filter(lambda container: container.get_id() == cont_id, source)
        if not len(match_list):
            return None

        # make sure container IDs are unique
        assert(len(match_list) == 1)
        return match_list[0]

    def get_dest_container(self, cont_id):
        dest = self.__context.get_project().get_dest()
        match_list = filter(lambda container: container.get_id() == cont_id, dest)
        if not len(match_list):
            return None

        # make sure container IDs are unique
        assert(len(match_list) == 1)
        return match_list[0]

    def get_dest_container_type(self, cont_id):
        cont = self.get_dest_container(cont_id)
        if not cont:
            return CONTAINER_UNKNOWN
        return cont.get_type()

    def get_source_title(self, cont_id, title_id):
        src_cont = self.get_source_container(cont_id)
        if not src_cont:
            return None

        return self.__get_title(src_cont, title_id)

    def get_dest_title(self, cont_id, title_id):
        dest_cont = self.get_dest_container(cont_id)
        if not dest_cont:
            return None

        return self.__get_title(dest_cont, title_id)

    def get_lang_name(self, iso_codes, language):
        ui_lang_name = _('Unknown')
        lang = iso_codes.find_iso_639_1(language)
        if lang:
            ui_lang_name = lang.get_name()
        return ui_lang_name

    def get_audio_list(self, source_container_id):
        result = []

        # pick just the streams which belong to this source media
        dest = self.get_context().get_dest()
        for container in dest:
            for title in container:
                audio_list = title.get_audio()
                for audio in audio_list:
                    source_audio = audio.get_source()
                    source_id = source_audio.get_container_id()
                    if source_id != source_container_id:
                        continue

                    result.append(audio)
        return result

    def get_video_list(self, source_container_id):
        result = []

        # pick just the streams which belong to this source media
        dest = self.get_context().get_dest()
        for container in dest:
            for title in container:
                video_list = title.get_video()
                for video in video_list:
                    source_video = video.get_source()
                    source_id = source_video.get_container_id()
                    if source_id != source_container_id:
                        continue

                    result.append(video)
        return result

    def get_subtitle_list(self, source_container_id):
        result = []

        # pick just the streams which belong to this source media
        dest = self.get_context().get_dest()
        for container in dest:
            for title in container:
                subtitle_list = title.get_subtitles()
                for subtitle in subtitle_list:
                    source_subtitle = subtitle.get_source()
                    source_id = source_subtitle.get_container_id()
                    if source_id != source_container_id:
                        continue

                    result.append(subtitle)
        return result

    def get_commands(self):
        return self.__commands

    def create_commands(self):
        if not self.__create_callback:
            return False

        return self.__create_callback(self.__commands)

    def get_ui_name(self):
        return self.__name

    # helpers used by the derived classes

    def compute_scale_expand(self, video, crop_values, aspect_dst, resolution):
        result = ''

        if not isinstance(video, Video):
            debug('Worker.compute_scale_expand()', 'video object type error', error)
            return result

        aspect_src = video.get_aspect()
        if aspect_dst == 'original':
            aspect_dst = aspect_src

        width_source = video.get_width()
        height_source = video.get_height()

        if resolution == 'original':
            width_target = width_source
            height_target = height_source
        else:
            w, h = resolution.split('x')
            width_target = to_int(w)
            height_target = to_int(h)
            if width_target < 0 or height_target < 0:
                debug('WorkerRequant.__compute_scale_expand()', 'resolution not set', error)
                return aspect_dst, result

        height_crop = height_source
        width_crop = width_source
        shift_x = -1
        shift_y = -1
        if crop_values and len(crop_values) == 4 and crop_values[0] > 0 and crop_values[1] > 0:
            width_crop, height_crop, shift_x, shift_y = crop_values
        else:
            print 'invalid crop'

        aspect_target = 16.0 / 9.0
        if aspect_dst == '4/3':
            aspect_target = 4.0 / 3.0

        aspect_source = 16.0 / 9.0
        if aspect_src == '4/3':
            aspect_source = 4.0 / 3.0

        width_expand = (width_target  * aspect_source) / aspect_target
        height_expand = height_target

        # let the width consume the whole screen and adapt the height
        width_scale = width_expand
        height_scale = (((((float(height_crop) * 100.0) / float(height_source)) * height_target) / 100.0) / aspect_source) * aspect_target

        # make all values multiple of 16
        width_crop = self.__align(width_crop)
        height_crop = self.__align(height_crop)
        width_scale = self.__align(width_scale)
        height_scale = self.__align(height_scale)
        width_expand = self.__align(width_expand)
        height_expand = self.__align(height_expand)

        result = ''
        if width_crop != width_source or height_crop != height_source:
            # todo: shift_x/y might be incorrect because width/height might have been altered
            # not by much though
            if shift_x > -1 and shift_y > -1:
                result = 'crop=%d:%d:%d:%d,' % (width_crop, height_crop, shift_x, shift_y)
            else:
                result = 'crop=%d:%d,' % (width_crop, height_crop)

        if width_scale != width_expand or height_scale != height_expand:
            result += ('scale=%d:%d,expand=%d:%d,' % (width_scale, height_scale, width_expand, height_expand))
        # if width_scale == width_expand and height_scale == height_expand, but
        # we still have to resize
        elif width_crop != width_scale or height_crop != height_scale:
            result += ('scale=%d:%d,' % (width_scale, height_scale))

        return aspect_dst, result

    def calc_vbitrate(self, disc_size, admin_size, occupied, seconds, vsize):
        shrink_needed = True
        vbitrate = self.__vbitrate(disc_size, admin_size, occupied, seconds)

        # mplayer and mencoder interpret values > 16000 as bits/second instead
        # of kbits/second
        if vbitrate > 16000:
            vbitrate = 16000

        # the space available for the viedo is larger than the video stream
        if vsize > -1 and (disc_size - admin_size - occupied) > vsize:
            shrink_needed = False

        return vbitrate, shrink_needed

    def get_shrink_method(self, video):
        disc_id = video.get_container_id()
        container = self.get_dest_container(disc_id)
        cfg = self.get_context().get_project().get_config()
        shrink_method = '1 pass'

        if container.get_type() == CONTAINER_DVD:
            shrink_method = self.__get_shrink_method_dvd(video)

        elif container.get_type() == CONTAINER_FILE:
            shrink_method = cfg.get_video_shrink_method_file()

        else:
            # todo
            pass

        return shrink_method

    def unlink_temp(self, file_list):
        for fname in file_list:
            if not fname or not len(fname):
                continue

            try:
                unlink(fname)
            except OSError:
                debug('Worker.unlink_temp()', 'cannot unlink %s' % fname, error)

        return


    def expand_chapter_list(self, ch_start, ch_end, outname):
        cfg = self.get_context().get_project().get_config()
        if not cfg.get_chapter_rip() or self.get_dest_container_type(0) != CONTAINER_DVD:
            return [(ch_start, ch_end, outname)]

        return [(ch, ch, append_to_filename(outname, '-%d' % ch)) for ch in range(ch_start, ch_end + 1)]

    def get_name_for_chapters(self, ch_start, ch_end, outname):
        cfg = self.get_context().get_project().get_config()
        if not cfg.get_chapter_rip() or self.get_dest_container_type(0) != CONTAINER_DVD:
            return outname

        if ch_start == ch_end:
            return append_to_filename(outname, '-%d' % ch_start)

        assert(False)
        return outname

    def remove_stream_if_empty_file(self, stream, check_kind):
        cont_id = stream.get_container_id()
        title_id = stream.get_title_id()
        title = self.get_dest_title(cont_id, title_id)

        if check_kind == EMPTY_FILE:
            filter_func = self.__check_empty
        elif check_kind == EMPTY_XML_FILE:
            filter_func = self.__check_empty_xml
        else:
            debug('Worker.remove_stream_if_empty_file()', 'unknown function type', error)
            return False

        # remove streams which does not contain data
        if isinstance(stream, Video):
            result = filter(filter_func, title.get_video())

            title.reset_video()
            for vstream in result:
                title.add_video(vstream)

        elif isinstance(stream, Audio):
            result = filter(filter_func, title.get_audio())

            title.reset_audio()
            for astream in result:
                title.add_audio(astream)

        elif isinstance(stream, Subtitle):
            result = filter(filter_func, title.get_subtitles())

            title.reset_subtitles()
            for sstream in result:
                title.add_subtitle(sstream)

        else:
            debug('Worker.remove_stream_if_empty_file()', 'unknown stream type', error)
            return False

        return True

    # -- helpers --

    def __align(self, number):
        if number % 16 != 0:
            number = to_int(number / 16) * 16
        return to_int(number)

    def __vbitrate(self, disc_size, admin_size, occupied, seconds):
        return (disc_size - admin_size - occupied) / seconds / 125.0

    def __get_shrink_method_dvd(self, video):
        cfg = self.get_context().get_project().get_config()
        shrink_method = cfg.get_video_shrink_method_dvd()

        if shrink_method == 'requantize':
            resol = cfg.get_video_resolution_dvd()
            w, h = resol.split('x')
            tw = to_int(w)
            th = to_int(h)
            sw = video.get_width()
            sh = video.get_height()

            saspect = video.get_aspect()
            taspect = cfg.get_video_aspect_dvd()
            if taspect == 'original':
                taspect = saspect

            # we cannot requantize if resolution or aspect change should be
            # made
            if taspect != saspect or tw != sw or th != sh:
                shrink_method = '1 pass'

        return shrink_method

    def __get_title(self, container, title_id):
        match_list = filter(lambda title: title.get_id() == title_id, container)
        if not len(match_list):
            return None

        # make sure title IDs are unique inside a container
        assert(len(match_list) == 1)
        return match_list[0]

    def __check_empty(self, stream):
        fname = stream.get_filename()

        if isinstance(stream, Subtitle):
            fname = set_extension(fname, 'sub')

        if does_path_exist(fname) and file_size(fname) > 0:
            return True

        return False

    def __check_empty_xml(self, stream):
        dest_dir = self.get_context().get_project().get_dest_dir()
        xml_fname = build_path([dest_dir, stream.get_filename()])

        if number_lines(xml_fname) > 4: # todo dirty check: lines > 4
            return True

        return False

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
