# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.worker import Worker, EMPTY_FILE
from winkirip.command_ui import UICommand
from winkirip.container import CONTAINER_DVD, CONTAINER_FILE, CONTAINER_UNKNOWN
from winkirip.container_audio import AUDIO_PCM, AUDIO_AC3, AUDIO_OGG
from winkirip.container_audio import AUDIO_MP2, AUDIO_MP3, AUDIO_FLAC
from winkirip.constructor_audio_dump import ConstructorAudioPcm, ConstructorAudioCopy
from winkirip.lib import debug, error
import re

class WorkerAudio(Worker):
    def __init__(self, context, source_id):
        self.__source_container_id = source_id

        Worker.__init__(self, context, self.__create, _('Extract audio'))
        self.__create_log_text = ''
        self.__exec_log_text = ''
        # match for mplayers output
        self.__regex = re.compile('A:\ +\d+\.\d+\ +V:\ +\d+\.\d+')
        return

    # -- helpers --

    def __create(self, command_list):
        iso_codes = self.get_context().get_iso_codes()

        alist = self.get_audio_list(self.__source_container_id)
        force_pcm = self.__get_force_pcm(alist)

        for audio in alist:
            fname, lname = self.__get_names(audio, force_pcm, iso_codes)

            disc_id = audio.get_container_id()
            title_id = audio.get_title_id()
            ui_templ = _('Extracting %s audio stream (disk %d :: title %d)')
            ui_text = ui_templ % (lname, disc_id + 1, title_id + 1)

            ui_command = UICommand(self.__exec, ui_text, True)
            ui_command.set_create_log_callback(self.__create_log)
            ui_command.set_exec_log_callback(self.__exec_log)
            ui_command.set_parse_callback(self.__parse_output)
            ui_command.set_data([audio, fname, force_pcm])
            command_list.append(ui_command)

        if len(alist) > 0:
            ui_text = _('Verifying audio streams')
            ui_command = UICommand(self.__exec_remove_file, ui_text, True)
            ui_command.set_exec_log_callback(self.__exec_log)
            ui_command.set_data([audio])
            command_list.append(ui_command)

        return True

    def __exec(self, command_data, command):
        assert(len(command_data) == 3)
        audio, outname, force_pcm = command_data

        source_audio = audio.get_source()
        cont_id = source_audio.get_container_id()
        title_id = source_audio.get_title_id()
        source_container = self.get_source_container(cont_id)
        source_title = self.get_source_title(cont_id, title_id)
        device = source_container.get_device()
        source_type = source_container.get_type()

        cont_id = audio.get_container_id()
        title_id = audio.get_title_id()
        title = self.get_dest_title(cont_id, title_id)
        ch_start = title.get_first_chapter_id()
        ch_end = title.get_last_chapter_id()

        result = self.__extract_audio(command, ch_start, ch_end, device, source_title, source_type, source_audio, outname, force_pcm)

        if result:
            self.__update_audio(audio, outname, force_pcm)
        else:
            command.set_error(_('You may get help in the winki forum located at http://www.winki-the-ripper.de'))

        return result

    def __extract_audio(self, command, ch_start, ch_end, device, source_title, source_type, source_audio, outname, force_pcm):
        tool_checker = self.get_context().get_tool_checker()
        chapt_info = self.expand_chapter_list(ch_start, ch_end, outname)

        for start_ch, end_ch, fname_ch in chapt_info:
            constr = None
            if force_pcm:
                constr = ConstructorAudioPcm(tool_checker)
            else:
                constr = ConstructorAudioCopy(tool_checker)

            if not constr:
                debug('WorkerAudio.__extract_audio()', 'command construction failed 0', error)
                return False

            constr.set_source(device, source_title.get_id())
            constr.set_source_type(source_type)
            constr.set_chapters(start_ch, end_ch)
            constr.set_outname(fname_ch)
            constr.set_aid(source_audio.get_id())

            success, shell_cmd = constr.construct()
            if not success:
                debug('WorkerAudio.__extract_audio()', 'command construction failed 1', error)
                command.set_error(_('Command construction failed'))
                return False

            self.__exec_log_text += ('%s\n' % shell_cmd)
            if not command.run_command(shell_cmd):
                return False

        return True

    def __get_force_pcm(self, alist):
        cfg = self.get_context().get_config()
        dest_format = cfg.get_audio_format_file()
        ctype = CONTAINER_UNKNOWN

        for audio in alist:
            disc_id = audio.get_container_id()
            dest_container = self.get_dest_container(disc_id)
            if dest_container:
                ctype = dest_container.get_type()
                if ctype == CONTAINER_DVD:
                    dest_format = cfg.get_audio_format_dvd()
                elif ctype == CONTAINER_FILE:
                    dest_format = cfg.get_audio_format_file()
                else:
                    # todo
                    pass

            # dest format is pcm
            if dest_format == AUDIO_PCM:
                return True

            # format conversion forces intermediate pcm file
            if dest_format != audio.get_format():
                return True

            # DVD chapter ripping: without going over pcm only the audio stream
            # for first chapter containes valid ac3 data
            if ctype == CONTAINER_DVD and cfg.get_chapter_rip():
                return True

            # crude approach: if the source title should be splitted
            src = audio.get_source()
            source_title = self.get_source_title(src.get_container_id(), src.get_title_id())
            dest_title = self.get_dest_title(audio.get_container_id(), audio.get_title_id())
            source_chapters = source_title.get_chapters()
            dest_chapters = dest_title.get_chapters()
            if len(source_chapters) != len(dest_chapters):
                return True

        return False

    def __get_names(self, audio, force_pcm, iso_codes):
        disc_id = audio.get_container_id()
        title_id = audio.get_title_id()

        aid = audio.get_id()
        atype = audio.format2str()

        if force_pcm:
            atype = 'wav'

        fname = 'audio/disc%d_title%d_audio%d.%s' % (disc_id + 1, title_id + 1, aid, atype)
        lname = self.get_lang_name(iso_codes, audio.get_language())

        return fname, lname

    def __exec_remove_file(self, command_data, command):
        assert(len(command_data) == 1)
        audio, = command_data

        return self.remove_stream_if_empty_file(audio, EMPTY_FILE)

    def __update_audio(self, audio, fname, force_pcm):
        audio.set_filename(fname)
        #audio.set_id(apos + 1)
        if force_pcm:
            audio.set_format(AUDIO_PCM)
        return

    def __parse_output(self, line):
        return 0, 0

        # the next line produces very, very strange output...
        #print 'line [%s]' % line

        # try to match something like 'A:  22.8 V:  22.9 A-V'
        match = self.__regex.match(line)
        if match:
            items = match.group().split()
            progress = int(float(items[1]))
            #print progress
        else:
            progress = 0
            estimation = 0
            #print 'no match'
        return progress, estimation

    def __create_log(self, fobj):
        if not self.__create_log_text or not len(self.__create_log_text):
            fobj.write('no audio information yet\n')
            return

        fobj.write('%s' % self.__create_log_text)
        return

    def __exec_log(self, fobj):
        if not self.__exec_log_text or not len(self.__exec_log_text):
            fobj.write('no audio information yet\n')
            return

        fobj.write('%s' % self.__exec_log_text)
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
