# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.worker import Worker
from winkirip.command_ui import UICommand
from winkirip.lib import set_extension, debug, error
from winkirip.container import CONTAINER_DVD, CONTAINER_FILE
from winkirip.container_audio import AUDIO_UNKNOWN, AUDIO_PCM, AUDIO_AC3
from winkirip.container_audio import AUDIO_OGG, AUDIO_MP2, AUDIO_MP3, AUDIO_FLAC
from constructor_audio import ConstructorOgg, ConstructorMp3, ConstructorMp2
from constructor_audio import ConstructorAc3

class WorkerAudioCompress(Worker):
    def __init__(self, context, source_id):
        self.__source_container_id = source_id
        self.__tempfile = None

        Worker.__init__(self, context, self.__create, _('Compress audio'))
        self.__create_log_text = ''
        self.__exec_log_text = ''
        return

    # -- helpers --

    def __create(self, command_list):
        iso_codes = self.get_context().get_iso_codes()

        alist = self.get_audio_list(self.__source_container_id)

        for audio in alist:
            target_format = self.__get_target_audio_format(audio)
            disc_id = audio.get_container_id()
            title_id = audio.get_title_id()
            lname = self.get_lang_name(iso_codes, audio.get_language())
            ui_templ = _('Compressing %s audio stream (disk %d :: title %d)')
            ui_text = ui_templ % (lname, disc_id + 1, title_id + 1)

            ui_command = UICommand(self.__exec, ui_text, True)
            ui_command.set_create_log_callback(self.__create_log)
            ui_command.set_exec_log_callback(self.__exec_log)
            ui_command.set_parse_callback(self.__parse_output)
            ui_command.set_data([audio])
            command_list.append(ui_command)

            if target_format != AUDIO_PCM:
                ui_templ = _('Removing uncompressed %s audio stream (disk %d :: title %d)')
                ui_text =  ui_templ % (lname, disc_id + 1, title_id + 1)
                ui_command = UICommand(self.__unlink_temp, ui_text, True)
                ui_command.set_create_log_callback(self.__create_log)
                ui_command.set_exec_log_callback(self.__exec_log)
                ui_command.set_data([audio])
            command_list.append(ui_command)
        return True

    def __exec(self, command_data, command):
        assert(len(command_data) == 1)
        audio, = command_data

        # current and required format matches: nothing to do
        target_format = self.__get_target_audio_format(audio)
        if target_format == audio.get_format():
            return True

        cont_id = audio.get_container_id()
        cont_type = self.get_dest_container_type(cont_id)
        title_id = audio.get_title_id()
        title = self.get_dest_title(cont_id, title_id)
        in_file = audio.get_filename()
        out_file = set_extension(in_file, self.__get_target_ext(audio))
        ch_start = title.get_first_chapter_id()
        ch_end = title.get_last_chapter_id()

        result = True
        chapt_info = self.expand_chapter_list(ch_start, ch_end, in_file)
        for start_ch, end_ch, fname_ch in chapt_info:
            constr = self.__get_command_constructor(target_format, cont_type)
            if not constr:
                self.__exec_log_text += ('unknown format %d\n' % target_format)
                command.set_error(_('Unknown audio format'))
                return False

            out_fname_ch = set_extension(fname_ch, self.__get_target_ext(audio))
            constr.set_outname(out_fname_ch)
            constr.set_inname(fname_ch)

            success, shell_cmd = constr.construct()
            if not success:
                self.__exec_log_text += ('shell command construction failed\n')
                return False

            self.__exec_log_text += ('%s\n' % shell_cmd)

            if not command.run_command(shell_cmd):
                result = False
                break

        if result:
            audio.set_filename(out_file)
            audio.set_format(target_format)
            self.__tempfile = in_file
        else:
            self.__exec_log_text += ('shell command execution failed\n')
            command.set_error(_('You may get help in the winki forum located at http://www.winki-the-ripper.de'))

        return result

    def __get_target_audio_format(self, audio):
        cont_id = audio.get_container_id()
        cont_type = self.get_dest_container_type(cont_id)

        cfg = self.get_context().get_config()

        target_format = AUDIO_UNKNOWN
        if cont_type == CONTAINER_DVD:
            target_format = cfg.get_audio_format_dvd()
        elif cont_type == CONTAINER_FILE:
            target_format = cfg.get_audio_format_file()
        else:
            assert(False)

        return target_format

    def __get_target_ext(self, audio):
        target_format = self.__get_target_audio_format(audio)
        ext = ''
        if target_format == AUDIO_PCM:
            ext = 'wav'
        elif target_format == AUDIO_AC3:
            ext = 'ac3'
        elif target_format == AUDIO_MP2:
            ext = 'mp2'
        elif target_format == AUDIO_MP3:
            ext = 'mp3'
        elif target_format == AUDIO_OGG:
            ext = 'ogg'
        elif target_format == AUDIO_FLAC:
            ext = 'flac'
        else:
            assert(False)
        return ext

    def __get_command_constructor(self, target_format, cont_type):
        constr = None

        tool_checker = self.get_context().get_tool_checker()
        cfg = self.get_context().get_config()

        if target_format == AUDIO_OGG:
            constr = ConstructorOgg(tool_checker)
            constr.set_quality(cfg.get_ogg_quality())
            constr.set_resample('48000')

        elif target_format == AUDIO_MP3:
            constr = ConstructorMp3(tool_checker)
            constr.set_quality(cfg.get_mp3_quality())
            constr.set_resample('48')

        elif target_format == AUDIO_MP2:
            constr = ConstructorMp2(tool_checker)
            if cont_type == CONTAINER_DVD:
                constr.set_quality('384')
                constr.set_resample('48000')
            else:
                constr.set_quality(cfg.get_mp2_quality())

        elif target_format == AUDIO_AC3:
            constr = ConstructorAc3(tool_checker)
            if cont_type == CONTAINER_DVD:
                constr.set_quality('384')
                constr.set_resample('48000')
            else:
                constr.set_quality(cfg.get_ac3_quality())

        return constr

    def __unlink_temp(self, command_data, command):
        assert(len(command_data) == 1)
        audio, = command_data

        cont_id = audio.get_container_id()
        title_id = audio.get_title_id()
        title = self.get_dest_title(cont_id, title_id)
        ch_start = title.get_first_chapter_id()
        ch_end = title.get_last_chapter_id()

        unlink_list = []
        chapt_info = self.expand_chapter_list(ch_start, ch_end, self.__tempfile)
        for start_ch, end_ch, fname_ch in chapt_info:
            self.__exec_log_text += ('rm %s\n' % fname_ch)
            unlink_list.append(fname_ch)

        self.unlink_temp(unlink_list)
        return True

    def __parse_output(self, line):
        progress = 0
        estimation = 0
        return progress, estimation

    def __create_log(self, fobj):
        if not self.__create_log_text or not len(self.__create_log_text):
            fobj.write('no compress information yet\n')
            return

        fobj.write('%s' % self.__create_log_text)
        return

    def __exec_log(self, fobj):
        if not self.__exec_log_text or not len(self.__exec_log_text):
            fobj.write('no compress information yet\n')
            return

        fobj.write('%s' % self.__exec_log_text)
        self.__exec_log_text = ''
        return


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
