# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.worker import Worker
from winkirip.command_ui import UICommand
from winkirip.lib import move, to_int, debug, error
from winkirip.constructor_mpeg2 import ConstructorMpeg2DVD
from winkirip.constructor_rawvideo import ConstructorRawvideo
from container import CONTAINER_FILE

class WorkerShrinkDVD(Worker):
    def __init__(self, context):
        Worker.__init__(self, context, self.__create, _('Shrink video stream'))
        self.__tname = 'video/temp.vob'
        self.__passlog = 'divx2pass.log'
        self.__create_log_text = ''
        self.__exec_log_text = ''
        return

    # -- helpers --

    def __create(self, command_list):
        self.__create_log_text = 'video files:\n'

        cfg = self.get_context().get_project().get_config()
        resolution = cfg.get_video_resolution_dvd()
        w, h = resolution.split('x')
        target_width = to_int(w)
        target_height = to_int(h)
        if target_width < 0 or target_height < 0:
            debug('WorkerShrinkDVD.__create()', 'resolution not set', error)
            return False

        disc_nr = 0
        for cont in self.get_context().get_project().get_dest():
            title_nr = 0
            for title in cont:
                exec_func, ui_text, pass_count, force = self.__requant_info(disc_nr, title_nr, title)
                self.__create_log_text += ('%s\n' % ui_text)

                # todo: video[0]
                shrink_method = self.get_shrink_method(title.get_video()[0])

                # "requantize" or "1 pass"
                ui_command = UICommand(exec_func, ui_text, True)
                ui_command.set_create_log_callback(self.__create_log)
                ui_command.set_exec_log_callback(self.__exec_log)
                if shrink_method == '2 pass':
                    ui_command.set_data([disc_nr, title_nr, 1])
                else:
                    ui_command.set_data([disc_nr, title_nr, force])
                command_list.append(ui_command)

                # second pass
                if shrink_method == '2 pass':
                    ui_text = (_('Shrinking video stream (disk %d :: title %d)') % (disc_nr + 1, title_nr + 1))
                    ui_command = UICommand(exec_func, ui_text, True)
                    ui_command.set_create_log_callback(self.__create_log)
                    ui_command.set_exec_log_callback(self.__exec_log)
                    ui_command.set_data([disc_nr, title_nr, 2])
                    command_list.append(ui_command)

                # cleanup temp file
                ui_text = (_('Removing uncompressed video stream (disk %d :: title %d)') % (disc_nr + 1, title_nr + 1))
                ui_command = UICommand(self.__unlink_temp, ui_text, True)
                ui_command.set_create_log_callback(self.__create_log)
                ui_command.set_exec_log_callback(self.__exec_log)
                ui_command.set_data([disc_nr, title_nr])
                command_list.append(ui_command)

                audio_list = title.get_audio()
                sub_list = title.get_subtitles()
                do_filter = len(audio_list) > 0 or len(sub_list) > 0

                # mplex workaround: mplex cannot use the files produced by
                # mencoder which dvdauthor swallows without problems, so we
                # reextract the video stream and mplex will be happy again...

                # don't do it if there are no audio and no subtitles
                if do_filter and (force or shrink_method == '2 pass' or shrink_method == '1 pass'):
                    ui_text = (_('Filtering video stream (disk %d :: title %d)') % (disc_nr + 1, title_nr + 1))
                    ui_command = UICommand(self.__filter, ui_text, True)
                    ui_command.set_create_log_callback(self.__create_log)
                    ui_command.set_exec_log_callback(self.__exec_log)
                    ui_command.set_data([disc_nr, title_nr])
                    command_list.append(ui_command)


                title_nr += 1

            disc_nr += 1
        return True

    def __exec_shrink(self, command_data, command):
        template = 'cat %s | LC_ALL=C tcrequant -f %f > %s'

        assert(len(command_data) == 3)
        disc_nr, title_nr, force_reencode = command_data

        disc = self.get_context().get_dest()[disc_nr]
        title = disc[title_nr]
        video = title.get_video()[0]
        factor = video.get_quant_factor()

        if force_reencode:
            command.set_error(_('Cannot change aspect/resolution with tcrequant'))
            return False

        # no need to shrink at all
        if factor <= 1.0 or not video.get_shrink_needed():
            self.__exec_log_text += ('no need to shrink (factor %f; needed %s)\n' % (factor, video.get_shrink_needed()))
            return True

        result = True
        ch_start = title.get_first_chapter_id()
        ch_end = title.get_last_chapter_id()

        chapt_info = self.expand_chapter_list(ch_start, ch_end, video.get_filename())
        for start_ch, end_ch, fname_ch in chapt_info:
            tname = self.get_name_for_chapters(start_ch, end_ch, self.__tname)

            if not move(fname_ch, tname):
                self.__exec_log_text += 'shrink: cannot move %s to %s\n' % (fname_ch, tname)
                command.set_error(_('Cannot move %s to %s') % (fname_ch, tname))
                return False

            shell_cmd = template % (tname, factor, fname_ch)

            self.__exec_log_text += ('%s\n' % shell_cmd)
            if not command.run_command(shell_cmd):
                result = False
                break

        if not result:
            return False

        return True

    def __exec_1_pass(self, command_data, command):
        self.__exec_log_text = ''

        assert(len(command_data) == 3)
        disc_nr, title_nr, force_reencode = command_data

        disc = self.get_context().get_dest()[disc_nr]
        title = disc[title_nr]
        video = title.get_video()[0]
        crop = title.get_crop()
        
        if force_reencode:
            video.set_shrink_needed(True)

        tool_checker = self.get_context().get_project().get_tool_checker()
        cfg = self.get_context().get_project().get_config()
        aspect = cfg.get_video_aspect_dvd()
        resol = cfg.get_video_resolution_dvd()

        if resol != '720x576':
            aspect = '4/3'

        aspect, scale_expand = self.compute_scale_expand(video, crop, aspect, resol)

        if resol != '%dx%d' % (video.get_width(), video.get_height()):
            video.set_shrink_needed(True)

        # no need to shrink at all
        if not video.get_shrink_needed():
            self.__exec_log_text += 'no need to shrink 1\n'
            return True

        vbitrate = video.get_bitrate()
        if vbitrate < 0:
            self.__exec_log_text = ('video bitrate not set')
            command.set_error(_('Video bitrate not set'))
            return False

        ch_start = title.get_first_chapter_id()
        ch_end = title.get_last_chapter_id()

        chapt_info = self.expand_chapter_list(ch_start, ch_end, video.get_filename())
        result = True
        for start_ch, end_ch, fname_ch in chapt_info:
            tname = self.get_name_for_chapters(start_ch, end_ch, self.__tname)

            if not move(fname_ch, tname):
                self.__exec_log_text += 'requant: cannot move %s to %s\n' % (fname_ch, tname)
                command.set_error(_('Cannot move %s to %s') % (fname_ch, tname))
                return False

            constr = ConstructorMpeg2DVD(tool_checker)
            constr.set_source(tname)
            constr.set_source_type(CONTAINER_FILE)
            constr.set_outname(fname_ch)
            constr.set_aspect(aspect)
            constr.set_vbitrate(vbitrate)
            constr.set_scale_expand(scale_expand)
            constr.set_pass_count(-1)
            constr.set_pass_nr(1)

            success, shell_cmd = constr.construct()
            if not success:
                command.set_error(_('You may get help in the winki forum located at http://www.winki-the-ripper.de'))
                return False

            self.__exec_log_text += ('%s\n' % shell_cmd)

            if not command.run_command(shell_cmd):
                result = False
                break

        if result:
            video.set_aspect(aspect)
            w, h = resol.split('x')
            video.set_width(to_int(w))
            video.set_height(to_int(h))

        return result

    def __exec_2_pass(self, command_data, command):
        self.__exec_log_text = ''

        assert(len(command_data) == 3)
        disc_nr, title_nr, pass_nr = command_data

        disc = self.get_context().get_dest()[disc_nr]
        title = disc[title_nr]
        video = title.get_video()[0]
        crop = title.get_crop()
        
        cfg = self.get_context().get_project().get_config()
        aspect = cfg.get_video_aspect_dvd()
        resol = cfg.get_video_resolution_dvd()

        if resol != '720x576':
            aspect = '4/3'
        aspect, scale_expand = self.compute_scale_expand(video, crop, aspect, resol)

        if resol != '%dx%d' % (video.get_width(), video.get_height()):
            video.set_shrink_needed(True)

        # no need to shrink at all
        if not video.get_shrink_needed():
            self.__exec_log_text += 'no need to shrink 2\n'
            return True

        vbitrate = video.get_bitrate()
        if vbitrate < 0:
            self.__exec_log_text('video bitrate not set')
            command.set_error(_('Video bitrate not set'))
            return False

        tool_checker = self.get_context().get_project().get_tool_checker()

        ch_start = title.get_first_chapter_id()
        ch_end = title.get_last_chapter_id()

        chapt_info = self.expand_chapter_list(ch_start, ch_end, video.get_filename())
        result = True
        for start_ch, end_ch, fname_ch in chapt_info:
            tname = self.get_name_for_chapters(start_ch, end_ch, self.__tname)
            lname = self.get_name_for_chapters(start_ch, end_ch, self.__passlog)

            if pass_nr == 1:
                if not move(fname_ch, tname):
                    self.__exec_log_text += 'requant: cannot move %s to %s\n' % (fname_ch, tname)
                    command.set_error(_('Cannot move %s to %s') % (fname_ch, tname))
                    return False

                fname_ch = '/dev/null'

            # rename back the log file from the first pass to the name expected by mencoder
            if pass_nr == 2 and lname != self.__passlog:
                self.__exec_log_text += 'mv %s %s\n' % (lname, self.__passlog)
                if not move(lname, self.__passlog):
                    result = False
                    break

            constr = ConstructorMpeg2DVD(tool_checker)
            constr.set_source(tname)
            constr.set_source_type(CONTAINER_FILE)
            constr.set_outname(fname_ch)
            constr.set_aspect(aspect)
            constr.set_vbitrate(vbitrate)
            constr.set_scale_expand(scale_expand)
            constr.set_pass_count(2)
            constr.set_pass_nr(pass_nr)

            success, shell_cmd = constr.construct()
            if not success:
                command.set_error(_('You may get help in the winki forum located at http://www.winki-the-ripper.de'))
                return False

            self.__exec_log_text += ('%s\n' % shell_cmd)

            if not command.run_command(shell_cmd):
                result = False
                break

            # protect the first pass log file from overwriting in chapter mode
            if pass_nr == 1 and lname != self.__passlog:
                self.__exec_log_text += 'mv %s %s\n' % (self.__passlog, lname)
                if not move(self.__passlog, lname):
                    result = False
                    break

        if result and pass_nr == 2:
            video.set_aspect(aspect)
            w, h = resol.split('x')
            video.set_width(to_int(w))
            video.set_height(to_int(h))

        return result

    def __unlink_temp(self, command_data, command):
        assert(len(command_data) == 2)
        disc_nr, title_nr = command_data

        disc = self.get_context().get_dest()[disc_nr]
        title = disc[title_nr]
        ch_start = title.get_first_chapter_id()
        ch_end = title.get_last_chapter_id()

        chapt_info = self.expand_chapter_list(ch_start, ch_end, self.__tname)
        temp_files = []

        for start_ch, end_ch, fname_ch in chapt_info:
            self.__exec_log_text = ('rm %s\n' % fname_ch)
            temp_files.append(fname_ch)

            lname = self.get_name_for_chapters(start_ch, end_ch, self.__passlog)
            temp_files.append(lname)

        self.unlink_temp(temp_files)
        return True

    def __filter(self, command_data, command):
        self.__exec_log_text = ''

        assert(len(command_data) == 2)
        disc_nr, title_nr = command_data

        disc = self.get_context().get_dest()[disc_nr]
        title = disc[title_nr]
        video = title.get_video()[0]

        tool_checker = self.get_context().get_project().get_tool_checker()

        ch_start = title.get_first_chapter_id()
        ch_end = title.get_last_chapter_id()

        temp_files = []
        result = True

        chapt_info = self.expand_chapter_list(ch_start, ch_end, video.get_filename())
        for start_ch, end_ch, fname_ch in chapt_info:
            tname = self.get_name_for_chapters(start_ch, end_ch, self.__tname)
            temp_files.append(tname)

            if not move(fname_ch, tname):
                self.__exec_log_text += 'filter: cannot move %s to %s\n' % (fname_ch, tname)
                command.set_error(_('Cannot move %s to %s') % (fname_ch, tname))
                return False

            constr = ConstructorRawvideo(tool_checker)
            constr.set_source(tname)
            constr.set_source_type(CONTAINER_FILE)
            constr.set_outname(fname_ch)

            success, shell_cmd = constr.construct()
            if not success:
                command.set_error(_('You may get help in the winki forum located at http://www.winki-the-ripper.de'))
                return False

            self.__exec_log_text += ('%s\n' % shell_cmd)

            if not command.run_command(shell_cmd):
                result = False
                break

        if not result:
            command.set_error(_('You may get help in the winki forum located at http://www.winki-the-ripper.de'))
        else:
            self.unlink_temp(temp_files)

        return result

    def __requant_info(self, disc_nr, title_nr, title):
        force_reencode = False
        video = title.get_video()[0]

        cfg = self.get_context().get_project().get_config()
        shrink_method = cfg.get_video_shrink_method_dvd()
        target_aspect = cfg.get_video_aspect_dvd()

        resolution = cfg.get_video_resolution_dvd()
        w, h = resolution.split('x')
        target_width = to_int(w)
        target_height = to_int(h)

        pass_count = -1

        # honor user request if possible
        exec_func = self.__exec_shrink
        ui_text = (_('Requantifying video stream (disk %d :: title %d)') % (disc_nr + 1, title_nr + 1))
        if shrink_method == '1 pass':
            exec_func = self.__exec_1_pass
            ui_text = (_('Shrinking video stream (disk %d :: title %d)') % (disc_nr + 1, title_nr + 1))
            pass_count = 1
        elif shrink_method == '2 pass':
            exec_func = self.__exec_2_pass
            ui_text = (_('Creating video index (disk %d :: title %d)') % (disc_nr + 1, title_nr + 1))
            pass_count = 2

        # requantify not possible if resize is needed
        orig_width = video.get_width()
        orig_height = video.get_height()
        if shrink_method == 'requantize' and (orig_width != target_width or orig_height != target_height):
            shrink_method = '1 pass'
            exec_func = self.__exec_1_pass
            ui_text = (_('Shrinking video stream (disk %d :: title %d)') % (disc_nr + 1, title_nr + 1))
            pass_count = 1
            force_reencode = True

        # requantify not possible if aspect change is needed
        elif shrink_method == 'requantize' and target_aspect != 'original' and target_aspect != video.get_aspect():
            shrink_method = '1 pass'
            exec_func = self.__exec_1_pass
            ui_text = (_('Shrinking video stream (disk %d :: title %d)') % (disc_nr + 1, title_nr + 1))
            pass_count = 1
            force_reencode = True

        return exec_func, ui_text, pass_count, force_reencode


    def __parser_callback(self, ui_command, text):
        progress = 0.0
        estimation = None
        return progress, estimation

    def __create_log(self, fobj):
        if not len(self.__create_log_text):
            fobj.write('no shrink information yet\n')
            return
            
        fobj.write(self.__create_log_text)
        return

    def __exec_log(self, fobj):
        if not len(self.__exec_log_text):
            fobj.write('no shrink information yet\n')
            return
            
        fobj.write(self.__exec_log_text)
        return


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
