# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.worker import Worker, EMPTY_FILE, EMPTY_XML_FILE
from winkirip.command_ui import UICommand
from winkirip.container import CONTAINER_DVD, CONTAINER_FILE
from winkirip.container_subtitle import SUBTITLE_IMGFILE, SUBTITLE_VOBSUB
from winkirip.container_subtitle import SUBTITLE_UNKNOWN
from winkirip.lib import debug, error, set_extension
from winkirip.lib import file_size, number_lines, build_path
from winkirip.constructor_subtitle import ConstructorSubtitleDvd
from winkirip.constructor_subtitle import ConstructorSubtitleVobsub

class WorkerSubtitles(Worker):
    def __init__(self, context, source_id):
        self.__source_container_id = source_id

        Worker.__init__(self, context, self.__create, _('Extract subtitles'))
        self.__create_log_text = ''
        self.__exec_log_text = ''
        return

    # -- helpers --

    def __create(self, command_list):
        iso_codes = self.get_context().get_iso_codes()

        slist = self.get_subtitle_list(self.__source_container_id)

        for subtitle in slist:
            lname = self.get_lang_name(iso_codes, subtitle.get_language())

            disc_id = subtitle.get_container_id()
            title_id = subtitle.get_title_id()
            ui_templ = _('Extracting %s subtitle stream (disk %d :: title %d)')
            ui_text = ui_templ % (lname, disc_id + 1, title_id + 1)

            ui_command = UICommand(self.__exec, ui_text, True)
            ui_command.set_create_log_callback(self.__create_log)
            ui_command.set_exec_log_callback(self.__exec_log)
            ui_command.set_parse_callback(self.__parse_output)
            ui_command.set_data([subtitle])
            command_list.append(ui_command)

        if len(slist) > 0:
            ui_text = _('Verifying subtitles')
            ui_command = UICommand(self.__exec_remove_file, ui_text, True)
            ui_command.set_exec_log_callback(self.__exec_log)
            ui_command.set_data([subtitle])
            command_list.append(ui_command)

        return True

    def __exec(self, command_data, command):
        assert(len(command_data) == 1)
        subtitle, = command_data

        source_subtitle = subtitle.get_source()
        cont_id = source_subtitle.get_container_id()
        source_container = self.get_source_container(cont_id)
        device = source_container.get_device()
        source_type = source_container.get_type()

        cont_id = subtitle.get_container_id()
        title_id = subtitle.get_title_id()
        title = self.get_dest_title(cont_id, title_id)
        ch_start = title.get_first_chapter_id()
        ch_end = title.get_last_chapter_id()
        outname = self.__get_outname(subtitle)

        result, sformat = self.__extract_subtitle(command, ch_start, ch_end, subtitle, outname, device, source_subtitle, source_type)

        if result:
            container = self.get_dest_container(cont_id)
            if container.get_type() == CONTAINER_DVD:
                outname = set_extension(outname, 'xml')
            elif container.get_type() == CONTAINER_FILE:
                outname = set_extension(outname, 'idx')

            self.__update_subtitle(subtitle, outname, sformat)
        else:
            command.set_error(_('You may get help in the winki forum located at http://www.winki-the-ripper.de'))
            self.__exec_log_text += 'execution failed\n'

        return result

    def __extract_subtitle(self, command, ch_start, ch_end, subtitle, outname, device, source_subtitle, source_type):
        chapt_info = self.expand_chapter_list(ch_start, ch_end, outname)

        for start_ch, end_ch, fname_ch in chapt_info:
            constr, sformat = self.__get_command_constructor(subtitle)
            if not constr or not len(outname):
                debug('WorkerSubtitle.__extract_subtitle()', 'command construction failed 0', error)
                return False, None

            constr.set_source(device, source_subtitle.get_title_id())
            constr.set_source_type(source_type)
            constr.set_chapters(start_ch, end_ch)
            constr.set_outname(fname_ch)
            constr.set_sid(source_subtitle.get_id())

            success, shell_cmd = constr.construct()
            if not success:
                debug('WorkerSubtitle.__extract_subtitle()', 'command construction failed 1', error)
                command.set_error(_('Command construction failed'))
                return False, None

            self.__exec_log_text += ('%s\n' % shell_cmd)

            if not command.run_command(shell_cmd):
                return False, None

        return True, sformat

    def __get_outname(self, subtitle):
        cont_id = subtitle.get_container_id()
        title_id = subtitle.get_title_id()
        title = self.get_dest_title(cont_id, title_id)
        spos = title.get_subtitle_pos(subtitle.get_id())

        ftempl = 'subtitles/disc%d_title%d_subtitle%d'
        return ftempl % (cont_id + 1, title_id + 1, spos + 1)

    def __get_command_constructor(self, subtitle):
        constr = None
        sformat = SUBTITLE_UNKNOWN

        cont_id = subtitle.get_container_id()
        title_id = subtitle.get_title_id()
        title = self.get_dest_title(cont_id, title_id)
        ctype = self.get_dest_container_type(cont_id)
        palette = title.get_palette_file()

        tool_checker = self.get_context().get_tool_checker()

        if ctype == CONTAINER_DVD:
            constr = ConstructorSubtitleDvd(tool_checker)
            sformat = SUBTITLE_IMGFILE
            constr.set_palette(palette)

        elif ctype == CONTAINER_FILE:
            constr = ConstructorSubtitleVobsub(tool_checker)
            sformat = SUBTITLE_VOBSUB
            constr.set_spos(0) # todo

        return constr, sformat

    def __exec_remove_file(self, command_data, command):
        assert(len(command_data) == 1)
        subtitle, = command_data

        cont_id = subtitle.get_container_id()

        check_kind = EMPTY_FILE
        if self.get_dest_container_type(cont_id) == CONTAINER_DVD:
            check_kind = EMPTY_XML_FILE

        return self.remove_stream_if_empty_file(subtitle, check_kind)

    def __update_subtitle(self, subtitle, fname, sformat):
        # todo: needed?
        #subtitle.set_language(slang)
        subtitle.set_filename(fname)
        subtitle.set_format(sformat)
        return

    def __parse_output(self, text):
        progress = 0.0
        estimation = None
        return progress, estimation

    def __create_log(self, fobj):
        if not self.__create_log_text or not len(self.__create_log_text):
            fobj.write('no subtitles information yet\n')
            return

        fobj.write('%s' % self.__create_log_text)
        return

    def __exec_log(self, fobj):
        if not self.__exec_log_text or not len(self.__exec_log_text):
            fobj.write('no subtitles information yet\n')
            return

        fobj.write('%s' % self.__exec_log_text)
        return


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
